#!/usr/bin/env python3
#
#  This file is part of ProfileView.
#
#  ProfileView is free software: you can redistribute it and/or modify
#  it under the terms of the CeCILL 2.1 Licence
#
#  ProfileView is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
#  You should have received a copy of the Licence CeCILL 2.1 along
#  with ProfileView. If not, see <https://cecill.info/>.
#

import sys, os, argparse
import time
import gzip, pickle

from pv_utils import *
from hmm_utils import *

# a HMMER3 model is assumed
def parseHmmerModel( fh ):
    hmm = hmm_model()
    while True:
        line = fh.readline()
        if not line:
            break
        tokens = line.split()
        if len(tokens) == 0:
            continue
        elif tokens[0] == "NAME":
            hmm.name = tokens[1]
        elif tokens[0] == "LENG":
            hmm.length = int(tokens[1])
        elif tokens[0] == "NSEQ":
            hmm.nseq = int(tokens[1])
        elif tokens[0] == "HMM":
            fh.readline() # skip line after HMM: m->m  m->i  m->d  i->m  i->i  d->m  d->d
            stateCount = 1
            line = fh.readline()
            while line and not line.startswith("//"):
                tokens = line.split()
                if len(tokens) == 26 and stateCount == int(tokens[0]):
                    hmm.states.append([float(v) for v in tokens[1:21]])
                    stateCount += 1
                elif len(tokens) == 21 and tokens[0] == "COMPO":
                    hmm.bg_freq = [float(v) for v in tokens[1:21]]
                line = fh.readline()
            if line.startswith("//") and hmm.is_valid():
                hmm.states = [hmm.bg_freq] + hmm.states
                yield hmm
            hmm = hmm_model()


def main(argv=None):
    start = time.perf_counter()

    parser = argparse.ArgumentParser()
    parser.add_argument('--hmm-file', dest='hmmFile', type=str, help='hmm file path')
    parser.add_argument('--hmm-dir', dest='hmmDir', type=str, help='hmm directory path')
    parser.add_argument('--prefix', dest='outPrefix', default="hmmerDict", type=str, help='output prefix of generated dictionary')
    args = parser.parse_args()

    # parameters check
    validParameters = True;
    if args.hmmFile is None and args.hmmDir is None:
        validParameters = False
        print_error('either one of --hmm-file or --hmm-dir arguments must be set')
    if args.hmmFile is not None and args.hmmDir is not None:
        validParameters = False
        print_error('it is not possible to use both --hmm-file and --hmm-dir arguments')
    elif args.hmmFile is not None and not os.path.isfile(args.hmmFile):
        validParameters = False
        print_error('--hmm-file \"{}\" is not a file'.format(args.hmmFile))
    elif args.hmmDir is not None and not os.path.isdir(args.hmmDir):
        validParameters = False
        print_error('--hmm-dir \"{}\" is not a directory'.format(args.hmmDir))
    if not validParameters:
        return 1

    start = time.perf_counter()

    fileList = []
    if args.hmmFile is not None:
        fileList = [ args.hmmFile ]
    if args.hmmDir is not None:
        hmmdir = args.hmmDir[:-1] if args.hmmDir.endswith("/") else args.hmmDir
        fileList = [ '{}/{}'.format(hmmdir,f) for f in os.listdir(hmmdir) if f.endswith('.hmm') and os.path.isfile('{}/{}'.format(hmmdir,f)) ]

    print_status("parsing HMMER3 models");
    hmmDict = {}
    for fname in fileList:
        with open(fname,'r') as fh:
            for hmm in parseHmmerModel(fh):
                hmmDict[ hmm.name ] = hmm
                #print(str(hmm))

    hmmDictFileName = "{}.pgz".format(args.outPrefix)
    print_status("writing ProfileView's HMM dictionary to \"{}\"".format(hmmDictFileName))
    with gzip.GzipFile(hmmDictFileName, 'w') as hmmDictFile:
        pickle.dump( hmmDict, hmmDictFile )

    exec_time = time.perf_counter() - start
    print_status('execution time: {:.2f}s'.format(exec_time));

    return 0

# Check if the program is not being imported
if __name__ == "__main__":
    sys.exit(main())

